@@ -1,10 +1,5 @@
-# ovirt-imageio
-# Copyright (C) 2018 Red Hat, Inc.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
+# SPDX-FileCopyrightText: Red Hat, Inc.
+# SPDX-License-Identifier: GPL-2.0-or-later
 
 from collections import namedtuple
 from functools import partial
@@ -19,8 +14,9 @@ from . import nbd
 
 _modules = {
     "file": file,
-    "nbd": nbd,
+    "http": http,
     "https": http,
+    "nbd": nbd,
 }
 
 
